/**********************************************************************
   Copyright (C) Christopher Yeoh <cyeoh@samba.org> 2005
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**********************************************************************/

#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <assert.h>
#include <signal.h>

#include "Game.H"
#include "protocol.h"
#include "GameConfig.H"
#include "LogFile.H"

void createServer(int *serverFD, int *observerFD)
{
    struct sockaddr_in sa;
    int val = 1;
    int serverPort, observerPort;

    GameConfig::GetConfig("ObserverPort", &observerPort);
    GameConfig::GetConfig("ServerPort", &serverPort);

    LogFile::Log("Using observer port: %i", observerPort);
    LogFile::Log("Using server port: %i", serverPort);

    // Create port for server
    memset(&sa, 0, sizeof(struct sockaddr_in)); 
    sa.sin_family = AF_INET;
    sa.sin_port = htons(serverPort);
    sa.sin_addr.s_addr = INADDR_ANY;
    
    *serverFD = socket(AF_INET, SOCK_STREAM, 0);
    if (*serverFD<0) {
	perror("Could not create server socket");
	exit(1);
    }

    if (setsockopt(*serverFD, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
	perror("setsockopt failed\n");
	exit(1);
    }

    if (bind(*serverFD, (struct sockaddr *)&sa, sizeof(struct sockaddr_in)) < 0) { 
	perror("Could not bind to playing address for server\n");
	exit(1);
    } 

    listen(*serverFD, 4);

    // Create port for observers
    memset(&sa, 0, sizeof(struct sockaddr_in)); 
    sa.sin_family = AF_INET;
    sa.sin_port = htons(observerPort);
    sa.sin_addr.s_addr = INADDR_ANY;
    
    *observerFD = socket(AF_INET, SOCK_STREAM, 0);
    if (*observerFD<0) {
	perror("Could not create server socket");
	exit(1);
    }

    if (setsockopt(*observerFD, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
	perror("setsockopt failed\n");
	exit(1);
    }

    if (bind(*observerFD, (struct sockaddr *)&sa, sizeof(struct sockaddr_in)) < 0) { 
	perror("Could not bind to playing address for server\n");
	exit(1);
    } 

    listen(*observerFD, 4);

}

void gameLoop(int serverFD, int observerFD)
{
    int numPlayers;

    Game game(serverFD, observerFD);

    while (1) {
	// Allow people to connect
	numPlayers = game.PlayersJoining(time(NULL));
	assert(numPlayers>1);

	// Play game
	game.StartGame();

	// Reset game state
	game.Reset();
    }

}


int main(int argc, char *argv[])
{
    GameConfig::Initialise();
    char *homeDir = getenv("HOME");
    if (homeDir!=NULL) {
	string configFileName(homeDir);
	configFileName += "/.sc-config";

	GameConfig::ReadConfigFile(configFileName);
    } else {
	printf("No HOME environment variable set - not reading config file\n");
    }

    string logFile;
    if (!GameConfig::GetConfig("LogFile", &logFile)) {
	logFile = "scServer.log";
    }
    
    if (!LogFile::OpenLog(logFile)) {
	fprintf(stderr, "Unable to open log file: %s", logFile.c_str());
    }

    string logLevel;
    if (GameConfig::GetConfig("LogLevel", &logLevel)) {
	if (logLevel=="NORMAL") {
	    LogFile::SetLogLevel(LogFile::NORMAL);
	} else if (logLevel=="DEBUG1") {
	    LogFile::SetLogLevel(LogFile::DEBUG2);
	} else if (logLevel=="DEBUG2") {
	    LogFile::SetLogLevel(LogFile::DEBUG1);
	} else if (logLevel=="DEBUG3") {
	    LogFile::SetLogLevel(LogFile::DEBUG3);
	} else {
	    LogFile::Log("Unrecognised log level: %s\n", logLevel.c_str());
	}
    }

    // Ignore SIGPIPE
    if (signal(SIGPIPE,SIG_IGN) == SIG_ERR) {
	assert(0);
    }

    LogFile::Log("Spellcast server started");
    int serverFD;
    int observerFD;

    createServer(&serverFD, &observerFD);

    gameLoop(serverFD, observerFD);

    LogFile::Log("Server exiting");
}
